/*
Copyright 2022 cuisongliu@qq.com.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package cmd

import (
	"fmt"

	"github.com/spf13/cobra"

	"github.com/labring/sealos/pkg/apply"
)

var exampleGen = `
generate a cluster with multi images, specify masters and nodes:
    sealos gen labring/kubernetes:v1.24.0 labring/calico:v3.22.1 \
        --masters 192.168.0.2,192.168.0.3,192.168.0.4 \
        --nodes 192.168.0.5,192.168.0.6,192.168.0.7 --passwd xxx

specify server InfraSSH port:
  all servers use the same InfraSSH port：
    sealos gen labring/kubernetes:v1.24.0 --masters 192.168.0.2,192.168.0.3,192.168.0.4 \
        --nodes 192.168.0.5,192.168.0.6,192.168.0.7 --port 24 --passwd xxx
  different InfraSSH port numbers：
    sealos gen labring/kubernetes:v1.24.0 --masters 192.168.0.2,192.168.0.3:23,192.168.0.4:24 \
        --nodes 192.168.0.5:25,192.168.0.6:25,192.168.0.7:27 --passwd xxx
`

func newGenCmd() *cobra.Command {
	genArgs := &apply.RunArgs{
		Cluster: &apply.Cluster{},
		SSH:     &apply.SSH{},
	}
	var genCmd = &cobra.Command{
		Use:     "gen",
		Short:   "generate a Clusterfile",
		Long:    `generate a Clusterfile of the kubernetes cluster, which can be applied by 'sealos apply' command`,
		Example: exampleGen,
		RunE: func(cmd *cobra.Command, args []string) error {
			cluster, err := apply.NewClusterFromArgs(args, genArgs)
			if err != nil {
				return err
			}
			fmt.Println(cluster.String())
			return nil
		},
	}
	genArgs.RegisterFlags(genCmd.Flags())
	return genCmd
}

func init() {
	rootCmd.AddCommand(newGenCmd())
}
