# 基于 OSM 的路径规划服务

## 服务介绍

![17359201](https://user-images.githubusercontent.com/14962503/179479809-2fd169b8-d452-4b2a-ad6b-8004389b40a5.jpeg)

路径规划（routing）是业务中常用的一项通用服务，核心是为了解决点到点的距离和时间预估问题。在物流行业，外卖行业以及供应链行业中，应用尤为普遍。路径规划需要基于真实的地图信息，模拟计算两点之间的实际行驶路径，包括距离信息，耗时信息，道路的导航信息。由于受通用交通管制等因素，现实活动物体类型不一样，会有不同的导航路径，常见有：小汽车导航，步行导航，以及自行车导航等，完备的路径规划服务需要支持多种导航类型。

目前国内外市面上提供路径规划服务的厂商比较少，国内有高德，百度，国外有 google 和 here。对于中小型企业来说，使用上述商业服务通常面临很大的成本支出。普通规模的调用量而言，每年要花费几十万元的调用成本支出，这还不包括 QPS 和 RT 限制。通常日常企业使用场景并不追求超高导航精度，只要规划准确率大于 90%以上都能接受。

基于 openStreetMap 的路径规划服务是不二之选。采用开放性地图数据构建路径规划服务，可以帮助企业大幅降低服务成本，而且能降低业务对地图服务厂商的强依赖。对于需要高 QPS，低 RT，以及大规模矩阵计算的场景下，该服务是最适应不过的。

### 服务优势

#### 服务免费

1. SaaS 服务完全免费，可提供中国全境的路径导航服务；
1. 无 QPS 限制；
1. 无 RT 限制；

#### 支持私有云部署

与 sealos 深度整合，可提供一站式的私有云部署能力；

#### 高性能

1. SaaS 服务公网调用评估 RT 为 50ms，远小于其他服务厂商；
1. 本地镜像服务调用平均 RT 在 5ms 左右；

#### 矩阵计算

对于规划类应用场景，距离或者时间的矩阵计算通常是必须的。基于 OSM 的路径规划服务可提供强大的矩阵计算服务。矩阵规划经过我们长期的算法优化后，RT 大幅降低，可支持 10000\*10000 的超大矩阵计算。

#### 地图数据高频更新

对于 SaaS 服务而言，背后的地图数据我们会高频定期更新，每周会基于最新的 OSM 数据构建新的地图网络。

#### 地图精度高

基于中国地图数据，在 500KM 内随机采样，分别调用如下三种服务获取距离数据：

1. 高德导航；
1. 精确球面距离计算；
1. OSM 导航服务；

下图为三种服务计算出的距离曲线图：

![image](https://user-images.githubusercontent.com/14962503/179479910-f52ecd96-2cf2-4116-ac2f-93daa859f468.png)

可以看出球面直线计算的距离偏差很大，并且距离越大，偏差的距离绝对值越高。而 OSM 导航与高德导航非常接近。为了进一步分析 OSM 导航精确度，我们绘制了误差率曲线：

![image](https://user-images.githubusercontent.com/14962503/179479965-301fdc3f-154c-4f45-98c4-c2fefb05268d.png)

可以看出球面距离计算的准确率通常在 15%以上，短距离可以达到 25%。而 OSM 导航可以将准确率控制在 5%以下，距离越大准确率越高。能够满足业务绝大多数的使用场景。

## 安装

暂不提供免费版本镜像，如有需要请联系 sealos 或应用提供方

## API 调用

### 调用地址

[http://116.204.64.253:7002/avatarsolver-portal/route/navigate](http://116.204.64.253:7002/avatarsolver-portal/route/navigate)

请求类型：POST

Content-Type：application/json

### 详细参数说明

1. 请求参数

| 字段名              | 字段说明                                                                                                                                                                 |
| ------------------- | ------------------------------------------------------------------------------------------------------------------------------------------------------------------------ |
| key                 | API 调用秘钥<br />1. 一般用户请填写："91cb09e7-72b7-4094-839e-166bdc279e01"<br />1. 私人定制用户请联系客服获取秘钥<br />                                                 |
| gisStandardType     | 经纬度的标准类型：<br />1. GCJ02：中国标准，国内大多数地图提供商即采用这一标准，譬如：高德地图；<br />1. WGS84：国际标准，海外地图即采用这一标准，譬如：谷歌地图；<br /> |
| startPosition       | 始发地经纬度：<br />1. lon：经度<br />1. lat：纬度<br />                                                                                                                 |
| endPosition         | 目的地经纬度：<br />1. lon：经度<br />1. lat：纬度<br />                                                                                                                 |
| needRouteDetail     | 是否需要导航描述<br />true：需要<br />false：不需要                                                                                                                      |
| instructionLangType | 导航描述语言类型：<br />1. 中文<br />1. 日文<br />1. 英文<br />                                                                                                          |
| drivingType         | 导航类型：<br />car：小汽车<br />motorcycle：摩托车<br />bike：自行车<br />foot：步行                                                                                    |

2. 返回参数

| 字段名           | 字段说明                                                                                                                                                                                                                                                        |
| ---------------- | --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| success          | 调用是否成功                                                                                                                                                                                                                                                    |
| errorCode        | 错误码<br />CB00001 : 业务异常，通常是业务逻辑异常导致，请仔细检测您的参数，请不要传入海外地图参数；<br />CB00002: 系统异常，通常是系统错误导致，请联系客服处理<br />CB00003: 无效参数，通常是入参格式不正确，请基于示例检测您的入参<br />CB0004: JSON 解析失败 |
| errorMsg         | 错误信息                                                                                                                                                                                                                                                        |
| appendMsg        | 额外错误说明                                                                                                                                                                                                                                                    |
| distance         | 距离，单位米                                                                                                                                                                                                                                                    |
| duration         | 时间，单位秒                                                                                                                                                                                                                                                    |
| wayPointList     | 途经点列表                                                                                                                                                                                                                                                      |
| instructionInfos | 导航描述列表                                                                                                                                                                                                                                                    |

### 调用示例

1. 调用请求

```json
{
  "key": "91cb09e7-72b7-4094-839e-166bdc279e01",
  "gisStandardType": "GCJ02",
  "startPosition": {
    "lon": 118.77087540907952,
    "lat": 31.97373869999062
  },
  "endPosition": {
    "lon": 118.81768217678243,
    "lat": 31.95568524518241
  },
  "needRouteDetail": true,
  "instructionLangType": 1,
  "drivingType": "car"
}
```

2. 调用返回结果

```json
{
  "success": true,
  "data": {
    "distance": 7397.578730590102,
    "duration": 414.191,
    "wayPointList": [
      {
        "lon": 118.77087540907952,
        "lat": 31.97373869999062
      },
      {
        "lon": 118.77171057566537,
        "lat": 31.976176940205473
      },
      {
        "lon": 118.77326941359988,
        "lat": 31.97631492889526
      },
      {
        "lon": 118.77365383511638,
        "lat": 31.976336469036127
      },
      {
        "lon": 118.77402934575777,
        "lat": 31.97623323886325
      },
      {
        "lon": 118.77417332400243,
        "lat": 31.976093172517697
      },
      {
        "lon": 118.77436007261561,
        "lat": 31.975995965839772
      },
      {
        "lon": 118.77452464223454,
        "lat": 31.975955860276223
      },
      {
        "lon": 118.77469570881107,
        "lat": 31.975949641330875
      },
      {
        "lon": 118.77486357986825,
        "lat": 31.975977615725707
      },
      {
        "lon": 118.77501906263807,
        "lat": 31.97603839044384
      },
      {
        "lon": 118.77518104424131,
        "lat": 31.976153447058323
      },
      {
        "lon": 118.7752968651657,
        "lat": 31.97630322835918
      },
      {
        "lon": 118.77553409235463,
        "lat": 31.976510906682044
      },
      {
        "lon": 118.77581226968336,
        "lat": 31.976568887130703
      },
      {
        "lon": 118.7769941751091,
        "lat": 31.9766658587903
      },
      {
        "lon": 118.78254647330775,
        "lat": 31.97718644551635
      },
      {
        "lon": 118.78243200179382,
        "lat": 31.97614799684331
      },
      {
        "lon": 118.78237848525576,
        "lat": 31.975340835292915
      },
      {
        "lon": 118.78237044026082,
        "lat": 31.974758977573288
      },
      {
        "lon": 118.78230844508978,
        "lat": 31.97408549317431
      },
      {
        "lon": 118.78212635281974,
        "lat": 31.973130089301975
      },
      {
        "lon": 118.78202202928054,
        "lat": 31.972748577423108
      },
      {
        "lon": 118.78190830889122,
        "lat": 31.972297290256538
      },
      {
        "lon": 118.78156908228598,
        "lat": 31.971371925354152
      },
      {
        "lon": 118.78151760480164,
        "lat": 31.97103005275097
      },
      {
        "lon": 118.78155184031081,
        "lat": 31.970720692004214
      },
      {
        "lon": 118.78165040903737,
        "lat": 31.970422268004405
      },
      {
        "lon": 118.78185606352544,
        "lat": 31.970120042896497
      },
      {
        "lon": 118.7821260552221,
        "lat": 31.969901535367825
      },
      {
        "lon": 118.78241313068946,
        "lat": 31.9697374963147
      },
      {
        "lon": 118.78276024300949,
        "lat": 31.969635281475313
      },
      {
        "lon": 118.78311594731677,
        "lat": 31.96959853907517
      },
      {
        "lon": 118.78345447356789,
        "lat": 31.96964189132111
      },
      {
        "lon": 118.78379300058931,
        "lat": 31.96973982726721
      },
      {
        "lon": 118.78407469101532,
        "lat": 31.969864911464263
      },
      {
        "lon": 118.78434489450434,
        "lat": 31.970019498162532
      },
      {
        "lon": 118.78468952238961,
        "lat": 31.970324670245738
      },
      {
        "lon": 118.7848984920958,
        "lat": 31.97048221528412
      },
      {
        "lon": 118.78557352537388,
        "lat": 31.97086181211764
      },
      {
        "lon": 118.78632344026819,
        "lat": 31.971131738150486
      },
      {
        "lon": 118.78675751373213,
        "lat": 31.97123383678684
      },
      {
        "lon": 118.7871488354916,
        "lat": 31.971349574345844
      },
      {
        "lon": 118.78728216858434,
        "lat": 31.971359522516714
      },
      {
        "lon": 118.78806308537274,
        "lat": 31.971492705072382
      },
      {
        "lon": 118.79028876908701,
        "lat": 31.971892185181137
      },
      {
        "lon": 118.7926778593224,
        "lat": 31.972386189718378
      },
      {
        "lon": 118.79482902883517,
        "lat": 31.972911846904275
      },
      {
        "lon": 118.79778600777526,
        "lat": 31.973791063767187
      },
      {
        "lon": 118.79887887867733,
        "lat": 31.974191706680948
      },
      {
        "lon": 118.79953442828861,
        "lat": 31.974465290864913
      },
      {
        "lon": 118.80071081006072,
        "lat": 31.974918850653744
      },
      {
        "lon": 118.80151796751592,
        "lat": 31.97517918090206
      },
      {
        "lon": 118.80244630007233,
        "lat": 31.975456107207723
      },
      {
        "lon": 118.80408699382062,
        "lat": 31.97614041065381
      },
      {
        "lon": 118.8047610052502,
        "lat": 31.976243527162392
      },
      {
        "lon": 118.80494277945557,
        "lat": 31.976234741772867
      },
      {
        "lon": 118.80527118477276,
        "lat": 31.97617453467006
      },
      {
        "lon": 118.80631267684645,
        "lat": 31.975926924938296
      },
      {
        "lon": 118.80692564706932,
        "lat": 31.97583325532514
      },
      {
        "lon": 118.80712358252501,
        "lat": 31.975825635418932
      },
      {
        "lon": 118.80734287559403,
        "lat": 31.975795085214056
      },
      {
        "lon": 118.8077646849731,
        "lat": 31.975682921333597
      },
      {
        "lon": 118.80792418347444,
        "lat": 31.97562387326085
      },
      {
        "lon": 118.80862631529756,
        "lat": 31.974989362079214
      },
      {
        "lon": 118.80903692015103,
        "lat": 31.97472474191828
      },
      {
        "lon": 118.80980559157352,
        "lat": 31.973895749817686
      },
      {
        "lon": 118.81481249896396,
        "lat": 31.96782374972137
      },
      {
        "lon": 118.81548078590373,
        "lat": 31.966940576603267
      },
      {
        "lon": 118.81610564845133,
        "lat": 31.966006086222606
      },
      {
        "lon": 118.81644411579667,
        "lat": 31.965437116221874
      },
      {
        "lon": 118.8170168640694,
        "lat": 31.964385437963365
      },
      {
        "lon": 118.81820423289753,
        "lat": 31.961968865773827
      },
      {
        "lon": 118.8183935866171,
        "lat": 31.961311576049564
      },
      {
        "lon": 118.81877825659062,
        "lat": 31.959736344443655
      },
      {
        "lon": 118.81884553797457,
        "lat": 31.958852826074867
      },
      {
        "lon": 118.81929467836665,
        "lat": 31.956238913954
      },
      {
        "lon": 118.81768217678243,
        "lat": 31.95568524518241
      }
    ],
    "instructionInfos": [
      "distance: 277m, 继续行驶到 铁心工业园路",
      "distance: 221m, 右转 到  软件大道",
      "distance: 844m, 在环岛内，使用2出口出环岛，进入软件大道",
      "distance: 345m, 右转",
      "distance: 151m, 保持右行 到  花神庙枢纽",
      "distance: 965m, 继续行驶到 花神庙枢纽",
      "distance: 1340m, 保持左行",
      "distance: 265m, 保持右行",
      "distance: 2832m, 保持右行",
      "distance: 152m, 右转",
      "distance: 0m, 终点到达"
    ]
  },
  "errorCode": null,
  "errorMsg": null,
  "appendMsg": null
}
```

## 服务支持

### 团队介绍

![image](https://user-images.githubusercontent.com/14962503/179480046-c085c528-bf8e-48ea-8a25-00563e40852f.png)

团队网址：[https://www.scienson.com/](https://www.scienson.com/)

### 服务说明

1. 私有化部署支持；
1. 可定制海外任意国家的地图导航服务，欢迎联系我们；
1. 可提供该服务的 7 \* 24 小时在线答疑

客服微信：

![weixin](https://user-images.githubusercontent.com/14962503/179480093-dc6fcfc4-fb02-4245-9155-6d0b7126a36f.jpg)
